package function.builtin.math;

import function.ArgumentValidator;
import function.FunctionNames;
import function.LispFunction;
import sexpression.Cons;
import sexpression.LispNumber;

import static sexpression.LispNumber.ZERO;

@FunctionNames({ "+" })
public class PLUS extends LispFunction {

    private ArgumentValidator argumentValidator;
    private MathFunction mathFunction;

    public PLUS(String name) {
        this.argumentValidator = new ArgumentValidator(name);
        this.argumentValidator.setEveryArgumentExpectedType(LispNumber.class);
        this.mathFunction = new MathFunction(number -> number, this::add);
    }

    @Override
    public LispNumber call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        return mathFunction.callTailRecursive(new Cons(ZERO, argumentList)).invoke();
    }

    private LispNumber add(LispNumber number1, LispNumber number2) {
        return new LispNumber(number1.getValue().add(number2.getValue()));
    }
}
